/*
   This iterator skips over every other item in the list.
*/
import java.util.Iterator;
import java.util.NoSuchElementException;

public class Lister2<E> implements Iterator<E>
{
   private Node<E> list;


   /**
   * Initialize a Lister2 with a specified linked list.
   * @param <CODE>head</CODE>
   *   a head reference for a linked list of objects
   * <dt><b>Postcondition:</b><dd>
   *   Subsequent activations of <CODE>next</CODE> will return the elements
   *   from this linked list, one after another. If the linked list changes
   *   in any way before all the elements have been returned, then the
   *   subsequent behavior of this <CODE>Lister2</CODE> is unspecified.
   **/
   public Lister2(Node<E> head)
   {
      list = head;
   }


   /**
   * Determine whether there are any more elements in this Lister2.
   * @param - none
   * @return
   *   <CODE>true</CODE> if there are more elements in this
   *   <CODE>Lister2</CODE>; <CODE>false</CODE> otherwise.
   **/
   public boolean hasNext( )
   {
      return (list != null);
   }


   /**
   * Retrieve the next element of this <CODE>Lister2</CODE>.
   * @param - none
   * <dt><b>Precondition:</b><dd>
   *   <CODE>hasMoreElements()</CODE> must be <CODE>true</CODE>.
   * @return
   *   The return value is the next element of this <CODE>Lister2</CODE>.
   *   Note that each element is only returned once, and then the
   *   <CODE>Lister2</CODE> automatically advances to the next element.
   * @exception NoSuchElementException
   *   Indicates that <CODE>hasMoreElements()</CODE> is <CODE>false<CODE>.
   **/
   public E next( )
   {
      E answer;

      if ( !hasNext() )
         throw new NoSuchElementException("The Lister2 is empty");

      answer = list.getData( );
      list = list.getLink( );
      if ( hasNext() )           // skip over an item in the list
         list = list.getLink( );

      return answer;
   }


   public void remove( )
   {
      throw new UnsupportedOperationException("Lister2.remove not allowed");
   }
}
