/******************************************************************************
* A Node<E> provides a generic node for a linked list. Each node
* contains a piece of data (which is a reference to an E object) and a link
* (which is a reference to the next node of the list). The reference stored
* in a node can be null.
*
* @note
*   Lists of nodes can be made of any length, limited only by the amount of
*   free memory in the heap. But beyond Integer.MAX_VALUE (2,147,483,647),
*   the answer from listLength is incorrect because of arithmetic
*   overflow.
******************************************************************************/
public class Node<E> implements Iterable<E>
{
   // Invariant of the Node class:
   //   1. Each node has one reference to an E Object, stored in the instance
   //      variable data.
   //   2. For the final node of a list, the link part is null.
   //      Otherwise, the  link part is a reference to the
   //      next node of the list.
   private E data;
   private Node<E> link;

   
   /**
    * Implement the Iterable<E> interface.
    */
   public java.util.Iterator<E> iterator()
   {
     return new Lister<E>(this);
   }
   
   
   /**
   * Initialize a node with a specified initial data and link to the next
   * node. Note that the initialLink may be the null reference,
   * which indicates that the new node has nothing after it.
   * @param initialData
   *   the initial data of this new node
   * @param initialLink
   *   a reference to the node after this new node--this reference may be null
   *   to indicate that there is no node after this new node.
   * @postcondition
   *   This node contains the specified data and link to the next node.
   **/
   public Node(E initialData, Node<E> initialLink)
   {
      data = initialData;
      link = initialLink;
   }

   /**
   * Accessor method to get the data from this node.
   * @param - none
   * @return
   *   the data from this node
   **/
   public E getData( )
   {
      return data;
   }


   /**
   * Accessor method to get a reference to the next node after this node.
   * @param - none
   * @return
   *   a reference to the node after this node (or the null reference if there
   *   is nothing after this node)
   **/
   public Node<E> getLink( )
   {
      return link;
   }


   /**
   * Modification method to set the data in this node.
   * @param newData
   *   the new data to place in this node
   * @postcondition
   *   The data of this node has been set to newData.
   *   This data is allowed to be null.
   **/
   public void setData(E newData)
   {
      data = newData;
   }


   /**
   * Modification method to set the link to the next node after this node.
   * @param newLink
   *   a reference to the node that should appear after this node in the linked
   *   list (or the null reference if there is no node after this node)
   * @postcondition
   *   The link to the node after this node has been set to newLink.
   *   Any other node (that used to be in this link) is no longer connected to
   *   this node.
   **/
   public void setLink(Node<E> newLink)
   {
      link = newLink;
   }
}